////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2020, NightShade Electronics - All rights reserved.
// 
// BSD 3-Clause License
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its
//    contributors may be used to endorse or promote products derived from
//    this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
/*
 * NightShade_Treo_Driver_Arduino.cpp
 *
 *  Created on: Oct 29, 2018
 *      Author: Aaron D. Liebold
 */

#include <stdint.h>
#include <Arduino.h>
#include <Wire.h>
#include <SPI.h>
#include <HardwareSerial.h>
#include "NightShade_Treo_Driver.h"

// Communication Objects
NightShade_Treo_TWI		Treo_TWI;
NightShade_Treo_GPIO	Treo_GPIO;
NightShade_Treo_SPI		Treo_SPI;
NightShade_Treo_UART	Treo_UART;

// ********* General Functions *********** //

void Treo_Delay(unsigned long delay_ms) {
	delay(delay_ms);
}

void Treo_Delayus(unsigned long delay_us) {
	delayMicroseconds(delay_us);
}

// *************************************** //

// **************** TWI ****************** //
NightShade_Treo_TWI::NightShade_Treo_TWI() {
	_device = 0;
	_port = 0;
	_slaveAddress = 0;
	_clockSpeed = 0;
}

int NightShade_Treo_TWI::begin(int port, uint8_t sevenBitAddress, uint32_t clockSpeed) {
	_device = port;
	_slaveAddress = sevenBitAddress;
	_clockSpeed = clockSpeed;
	Wire.begin();
	Wire.setClock(_clockSpeed);
	return 0;
}
int NightShade_Treo_TWI::write(uint8_t *buffer, int numberOfBytes, int sendStop) {
	Wire.beginTransmission(_slaveAddress);
	for (int x = 0; x < numberOfBytes; ++x)
		Wire.write(*(buffer + x));
	return Wire.endTransmission(sendStop);
}
int NightShade_Treo_TWI::read(uint8_t *buffer, int numberOfBytes, int sendStop) {
	int bytesReceived = Wire.requestFrom((int) _slaveAddress,
			(int) numberOfBytes, (int) sendStop);
	for (int x = 0; x < bytesReceived; ++x)
		*(buffer + x) = Wire.read();
	return (bytesReceived == numberOfBytes ? 0 : -1);
}
int NightShade_Treo_TWI::end() {
	return 0;
}
// ************** end TWI ************* //

// **************** GPIO ****************** //
NightShade_Treo_GPIO::NightShade_Treo_GPIO() {

}

void NightShade_Treo_GPIO::setup(int pin, int mode) {
	if (mode == GPIO_OUTPUT) {
		pinMode(pin, 1); // Set as output
	} else {
		pinMode(pin, 0); // Set as input
	}
}
void NightShade_Treo_GPIO::write(int pin, int state) {
	if (state == GPIO_HIGH) {
		digitalWrite(pin, 1); // Set pin state high
	} else {
		digitalWrite(pin, 0); // Set pin state low
	}
}
int NightShade_Treo_GPIO::read(int pin) {
	return digitalRead(pin);
}
// ************** end GPIO **************** //

// **************** SPI ****************** //
NightShade_Treo_SPI::NightShade_Treo_SPI() {
	_device = 0;
	_port = 0;
	_csPin = 0;
	_spiMode = 0;
	_bitsPerWord = 0;
	_clockSpeed = 0;
	_lsbFirst = 0;
}

// Arduino Due is not currently Supported

int NightShade_Treo_SPI::begin(int port, int chipSelectPin, uint32_t clockSpeed,
		int spiMode, int bitsPerWord, int lsbFirst) {
	_port = port;
	_csPin = chipSelectPin;
	_bitsPerWord = bitsPerWord;
	_clockSpeed = clockSpeed;

	if (lsbFirst) _lsbFirst = LSBFIRST;
	else _lsbFirst = MSBFIRST;

	switch (spiMode) {
	case 0:
		_spiMode = SPI_MODE0;
		break;
	case 1:
		_spiMode = SPI_MODE1;
		break;
	case 2:
		_spiMode = SPI_MODE2;
		break;
	case 3:
		_spiMode = SPI_MODE3;
		break;
	default:
//		_spiMode = SPI_MODE0;
		break;
	}

	Treo_GPIO.setup(_csPin, GPIO_OUTPUT);
	SPI.begin();
	if (lsbFirst) SPI.beginTransaction(
			SPISettings(_clockSpeed, LSBFIRST, _spiMode));
	else SPI.beginTransaction(SPISettings(_clockSpeed, MSBFIRST, _spiMode));
	return 0;
}
int NightShade_Treo_SPI::transfer(uint8_t *buffer, int numberOfBytes) {
	Treo_GPIO.write(_csPin, GPIO_LOW);
	SPI.transfer(buffer, numberOfBytes);
	Treo_GPIO.write(_csPin, GPIO_HIGH);
	return 0;
}
int NightShade_Treo_SPI::end() {
	SPI.endTransaction();
	SPI.end();
	return 0;
}
// ************** end SPI **************** //

// **************** UART ****************** //
NightShade_Treo_UART::NightShade_Treo_UART() {
	_port = 0;
	_device = 0;
	_baudrate = 0;
	_parity = 0;
	_dataBits = 0;
	_endBits = 0;
}

// Pairity: 0 - None, 1 - Odd, 2 - Even
// Databits: 5-8
// End bit(s): 1-2
void NightShade_Treo_UART::begin(int port, uint32_t baudrate, uint8_t dataBits,
		uint8_t parity, uint8_t endBits) {
	_port = port;
	_baudrate = baudrate;
	_parity = parity;
	_dataBits = dataBits;
	_endBits = endBits;

	uint8_t UART_Settings = (_dataBits - 5) * 2; // Databits
	if (_parity > 0) UART_Settings += (3 - _parity) << 8; // Pairity
	if (_endBits > 1) UART_Settings += 8; // End bits

	Serial.begin(_baudrate, UART_Settings);
}

int NightShade_Treo_UART::write(uint8_t *buffer, int numberOfBytes) {
	int ret = Serial.write(buffer, numberOfBytes);
	return (ret == numberOfBytes ? 0 : -1);
}

int NightShade_Treo_UART::read(uint8_t *buffer, int numberOfBytes) {
	int ret = Serial.readBytes(buffer, numberOfBytes);
//	for (int x = 0; x < numberOfBytes; ++x) {
//		while (!Serial.available())
//			;
//		*(buffer + x) = Serial.read();
//	}
	return (ret == numberOfBytes ? 0 : -1);
}

int NightShade_Treo_UART::end() {
	return 0;
}
// **************** end UART ****************** //
